import java.util.ArrayList;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;

public class ParallelMapPool<X,Y> extends Map<X,Y> {

	ParallelMapPool(ArrayList<X> source,
			Function<X,Y> func,
			ArrayList<Y> target) {
		super(source, func, target);
	}

	// This applicator class has access to
	// the two arrays
	public class Applicator implements Runnable {
		int loc;
		Applicator(int loc) {
			this.loc = loc;
		}

		public void run() {
			target.set(loc, func.apply(source.get(loc)));
		}
	}

	public void map() {
		ExecutorService pool = Executors.newCachedThreadPool();

		for (int i=0 ; i<source.size() ; i++) {
			pool.execute(new Applicator(i));
		}
		pool.shutdown();
		try {
			pool.awaitTermination(1, TimeUnit.DAYS);
		}
		catch (InterruptedException e) {			
		}
	}
	
	// This applicator class has access to
	// the source array
	public class Applicator1 implements Callable<Y> {
		int loc;
		Applicator1(int loc) {
			this.loc = loc;
		}

		public Y call() {
			return func.apply(source.get(loc)); 
		}
	}

	public void map1() {
		ExecutorService pool = Executors.newCachedThreadPool();
		ArrayList<Future<Y>> futures = new ArrayList<Future<Y>>(source.size());
		for (int i=0 ; i<source.size() ; i++) {
			futures.set(i,pool.submit(new Applicator1(i)));
		}
		
		for (int i=0 ; i<target.size() ; i++) {
			try {
			target.set(i,futures.get(i).get());
			}
			catch (InterruptedException e) {
				// Here we end up if this
				// thread was interrupted
				
				// You might want to wait again
			}
			catch (ExecutionException e) {
				// Here we end up if the 
				// execution of the thread had an exception
				
				// You might want to run it again
			}
		}
		pool.shutdownNow();
	}

	public class Applicator2 implements Callable<Y> {
		X val;
		Applicator2(X val) {
			this.val = val;
		}

		public Y call() {
			return func.apply(this.val); 
		}
	}

	public void map2() {
		ExecutorService pool = Executors.newCachedThreadPool();
		ArrayList<Future<Y>> futures = new ArrayList<Future<Y>>(source.size());
		for (int i=0 ; i<source.size() ; i++) {
			futures.set(i,pool.submit(new Applicator2(source.get(i))));
		}
		
		for (int i=0 ; i<source.size() ; i++) {
			try {
			target.set(i,futures.get(i).get());
			}
			catch (InterruptedException e) {
				// Here we end up if this
				// thread was interrupted
				
				// You might want to wait again
			}
			catch (ExecutionException e) {
				// Here we end up if the 
				// execution of the thread had an exception
				
				// You might want to run it again
			}
		}
		pool.shutdownNow();
	}

	public static void main(String[] args) {
		ArrayList<Integer> start = new ArrayList<Integer>(1000);
		ArrayList<Integer> target = new ArrayList<Integer>(1000);
		
		for (int i=0 ; i<start.size() ; i++) {
			start.set(i,1);
		}
		
		Function<Integer, Integer> f = (Integer x) -> x+1;
		
		ParallelMapPool<Integer,Integer> p = 
				new ParallelMapPool<Integer,Integer>(start,f,target);
		p.map2();
		
		for (int i=0 ; i< target.size() ; i++) {
			if (target.get(i) != 2) {
				System.out.println("Somethign went wrong");
			}
		}
		System.out.println("Finished");
	}
}
