import java.util.ArrayList;
import java.util.function.BinaryOperator;

public class ParallelReduce<X> extends Reduce<X> {

	ParallelReduce(ArrayList<X> source,
			BinaryOperator<X> func,
			X initial) {
		super(source, func, initial);
	}

	public class Applicator extends Thread {
		int start;
		int end;
		X initial;
		Applicator(int start, int end, X initial) {
			this.start = start;
			this.end = end;
			this.initial = initial;
		}

		public void run() {
			if (end - start > 1) {
				int mid = start + (end - start) / 2;
				Thread l = new Applicator(start,mid,initial);
				Thread r = new Applicator(mid,end,initial);
				l.start(); r.start();
				try {
					l.join(); r.join();
					source.set(start, func.apply(source.get(start), source.get(mid)));
				} catch (InterruptedException e) {}
			} else {
				source.set(start,func.apply(initial, source.get(start)));
			}
		}
	}

	public X reduce() {
		Applicator a = new Applicator(0,source.size(),initial);
		a.run();
		return source.get(0);
	}
	
	public static void main(String[] args)
	{
		ArrayList<String> arr = new ArrayList<String>(7);
		arr.add("a");
		arr.add("b");
		arr.add("c");
		arr.add("d");
		arr.add("e");
		arr.add("f");
		arr.add("g");

		String s= new String();
		BinaryOperator<String> f = (String a, String b) -> a+b;
		Reduce<String> r = new Reduce<String>(arr,f,s);
		System.out.println(r.reduce());
		System.out.println("blah");
		System.out.println(s);
		System.out.println("blah");

		ArrayList<Integer> arr1 = new ArrayList<Integer>(2);
		for (int i=1 ; i<12 ; i++) {
			arr1.add(i);
		}

		BinaryOperator<Integer> f1 = (Integer a, Integer b) -> a+b;
		Integer s1=0;
		Reduce<Integer> r1 = new Reduce<Integer>(arr1,f1,s1);
		System.out.println(r1.reduce());
		System.out.println("blah");
		System.out.println(s);
		System.out.println("blah");

	}

}
